#ifndef GENARRAY_H
#define GENARRAY_H

/* copyright 2021 university of freiburg */
/* author: Peter Thiemann <thiemann@informatik.uni-freiburg.de> */

/* generic dynamic array API */
/* arbitrary element type */

typedef struct _genarray genarray;

/* array functions: create/new, destroy/free, read, write */

/* new array of elements of size element_size all initialized to copies of default_value, returns NULL on error */
genarray *ga_new(size_t initial_size, size_t element_size, const void *default_value);

/* deallocate all memory associated with ga */
void ga_destroy(genarray *ga);

/* copies either value previously stored @i or the default given by ga_new into return_val (does not resize) */
void ga_read(const genarray *ga, size_t i, void *return_val);

/* write to arbitrary index i (resize if necessary). Returns current size or -1 on failure. */
int ga_write(genarray *ga, size_t i, const void *val);

#endif // GENARRAY_H
