#include <unistd.h>
#include "stdlib.h"

#include "tui.h"
#include "./vec.h"
#include "gamestate.h"

int main(void) {
  tui_init();
  Size2 win_size = tui_size();	/* do not resize the window!! */
  Size2 size = win_size;
  size.y--;			/* reserve one line for message */

  /* initialize model */
  game_state game = {
    { .headchar = 'o',
      .tailchar = 'x',
      .head_pos = { .x = size.x / 2, .y = size.y /2 },
      .direction = EAST,
      .crashed = false,
      .feed = 10,
      .len = 0,
      .tail = vec_new()
    }
  };

  char c = '!';
  size_t t = 0;

  while (1) {
    if (game.worm.crashed) {
      while (read_from_stdin() != 'q');
      break;
    }
    if (stdin_has_changed()) {
      c = read_from_stdin();
      if (c == 'q')
        break;
      switch (c) {
      case NORTH:
      case SOUTH:
      case EAST:
      case WEST:
	game.worm.direction = c;
      }
    }
    
    /* change model according to command and time step */
    /* make a copy of worm's head pos & save in tail */
    Size2 * head_pos_p = malloc (sizeof (Size2));
    head_pos_p->x = game.worm.head_pos.x;
    head_pos_p->y = game.worm.head_pos.y;
    vec_push (game.worm.tail, head_pos_p);
    if (game.worm.feed > 0) {
      game.worm.len++;
      game.worm.feed--;
    } else {
      // forget the last position of the worm's tail
      vec_remove (game.worm.tail, 0);
    }
    
    if (game.worm.direction == NORTH && game.worm.head_pos.y > 0) {
      game.worm.head_pos.y--;
    } else if (game.worm.direction == SOUTH && game.worm.head_pos.y < size.y -1) {
      game.worm.head_pos.y++;
    } else if (game.worm.direction == WEST && game.worm.head_pos.x > 0) {
      game.worm.head_pos.x--;
    } else if (game.worm.direction == EAST && game.worm.head_pos.x < size.x -1) {
      game.worm.head_pos.x++;
    } else {
      // worm runs into the boundary!!!
      game.worm.crashed = true;
    }

    /* did worm run into itself? */
    for (size_t i = 0; i < vec_length (game.worm.tail); i++) {
      Size2 ** tailpp = (Size2 **) vec_at (game.worm.tail, i);
      if (game.worm.head_pos.x == (*tailpp)->x
	  && game.worm.head_pos.y == (*tailpp)->y) {
	game.worm.crashed = true;
      }
    }
    

    tui_clear();

    /* model -> view */
    Cell* cell;
    /* the worm's tail */
    for (size_t i = 0; i < vec_length (game.worm.tail); i++) {
      Size2 ** head_pos_pp = (Size2 **) vec_at (game.worm.tail, i);
      cell = tui_cell_at ((*head_pos_pp)->x, (*head_pos_pp)->y);
      cell->content = game.worm.tailchar;
      cell->text_color = FG_HI_YELLOW;
      cell->background_color = BG_BLACK;
    }
    /* the worm's head */
    cell = tui_cell_at(game.worm.head_pos.x, game.worm.head_pos.y);
    cell->content = game.worm.headchar;
    cell->text_color = FG_HI_YELLOW;
    cell->background_color = BG_BLACK;

    char buf[255];
    sprintf(buf, "LENGTH: %4ld    FEED: %2ld    SPEED: %2d    %s",
            game.worm.len, game.worm.feed, 0, game.worm.crashed ? "CRASHED": "");
    tui_set_str_at(0, size.y, buf, FG_WHITE, BG_BLACK);

    tui_update();

    t += 1;

    usleep(100000);
  }

  tui_shutdown();

  return 0;
}
